/*
 * CONTRAT DE LICENCE DE LOGICIEL LIBRE CeCILL-B
 * https://cecill.info/licences/Licence_CeCILL-B_V1-fr.html
 */
package fr.unistra.sil.erp.back;

import fr.unistra.sil.erp.back.interceptor.api.ApiAuthenticationInterceptor;
import org.springframework.context.annotation.*;
import org.springframework.core.env.Environment;
import org.springframework.web.servlet.config.annotation.CorsRegistry;
import org.springframework.web.servlet.config.annotation.EnableWebMvc;
import org.springframework.web.servlet.config.annotation.InterceptorRegistry;
import org.springframework.web.servlet.config.annotation.WebMvcConfigurer;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * Main configuration file for the application.
 *
 * @author BEAUVAIS ANTOINE
 */
@Configuration
@EnableWebMvc
public class WebMvcConfig implements WebMvcConfigurer {

    /**
     * Autowired environment for retrieving properties.
     */
    private final Environment env;

    /**
     * API version.
     */
    public static final String API_VERSION = "v1";

    /**
     * API prefix.
     */
    public static final String API_PREFIX = "/api/";

    /**
     * Prefix for API calls.
     */
    public static final String API_FULL_PREFIX = API_PREFIX + API_VERSION;

    /**
     * API Mapping for retrieving all items.
     */
    public static final String MAPPING_ITEMS = API_FULL_PREFIX +
            "/item";

    /**
     * API Mapping for retrieving all categories.
     */
    public static final String MAPPING_CATEGORY = API_FULL_PREFIX +
            "/category";

    /**
     * API Mapping for submitting transactions.
     */
    public static final String MAPPING_SUBTRANSAC = API_FULL_PREFIX +
            "/transaction";

    /**
     * API Mapping for retrieving stocks.
     */
    public static final String MAPPING_STOCKS = API_FULL_PREFIX +
            "/stock";

    /**
     * Web mapping for the login page.
     */
    public static final String MAPPING_LOGIN = "/login";

    /**
     * Web mapping for the items list.
     */
    public static final String WEB_MAPPING_ITEMS = "/products";

    /**
     * @param env
     */
    public WebMvcConfig(Environment env) {
        this.env = env;
    }

    /**
     * Adds interceptors to the application.
     * <p>
     * Interceptors process HTTP requests before they reach their
     * attributed methods.
     * <p>
     * Currently, interceptors are only used for authentication in this
     * application.
     *
     * @param registry the Interceptor Registry to use for additions.
     */
    @Override
    public void addInterceptors(InterceptorRegistry registry) {
        registry.addInterceptor(new ApiAuthenticationInterceptor(this.env.getRequiredProperty("api.key"))).addPathPatterns(API_PREFIX + "**");
    }

    @Override
    public void addCorsMappings(CorsRegistry registry) {
        registry.addMapping("/**")
                .allowedOrigins("*")
                .allowedHeaders("*");
    }

    /**
     * SQL Connection object.
     */
    private Connection conn;

    /**
     * @return
     */
    @Bean
    public Connection connection() {
        if (this.conn == null) {
            try {
                conn = DriverManager.getConnection(env.getRequiredProperty("spring.datasource.url"));
            } catch (SQLException ex) {
                Logger.getLogger(this.getClass().getName()).log(
                        Level.SEVERE, "Failed to connect to SQLite file.", ex);
            }
        }
        return conn;
    }
}
